/*------------------------------------------------
				HTMLParser.cpp
--------------------------------------------------*/
#include "stdafx.h"
#include "HTMLParser.h"

/* Semantic Rutines */
int OnText(char c);                  // Al leer un prrafo
int OnEndTextByTag(char c);          // Al terminar de leer debido a un Tag
int OnEndTextByEOF(char c);          // Al terminar de leer debido a un EOF
int OnTag(char c);                   // Al leer un tag
int OnEndTag(char c);                // Al terminar de leer un Tag
int OnETag(char c);                  // Al leer un Etag </X>
int OnEndETag(char c);               // Al terminar de leer un Etag
int OnEOF(char c);                   // Al leer un End Of File

/*                                   EOF [0]           Letra [1]        Digito [2]       Blanc [3] < [4]   > [5]      / [6]             */
int newState[7][7] = {
				   /* Inicial [0] */ {END,             1,               1,               0,        4,      1,         1},
					 /* Texto [1] */ {END,             1,               1,               3,        2,      1,         1},
   /* Posible EndText por Tag [2] */ {END,             END,             END,             3,        2,      1,         END},
	/* Espacios descartados / [3] */ {END,             1,               1,               3,        2,      1,         1},
				/*Posible Tag [4] */ {END,             5,               5,               3,        1,      END,       6},
					   /* Tag [5] */ {END,             5,               5,               5,        5,      END,       5},
					  /* ETag [6] */ {END,             6,               6,               6,        6,      END,       6}};

/*                                   EOF [0]           Letra [1]        Digito [2]       Blanc [3] < [4]   > [5]      / [6]             */
int (*pfSemRut[7][7])(char) = {
				   /* Inicial [0] */ {OnEOF,           OnText,          OnText,          NULL,     OnText, OnText,    OnText},
					 /* Texto [1] */ {OnEndTextByEOF,  OnText,          OnText,          OnText,   OnText, OnText,    OnText},
   /* Posible EndText por Tag [2] */ {OnEndTextByEOF,  OnEndTextByTag,  OnEndTextByTag,  OnText,   OnText, OnText,    OnEndTextByTag},
	/* Espacios descartados / [3] */ {OnEndTextByEOF,  OnText,          OnText,          NULL,     OnText, OnText,    OnText},
				/*Posible Tag [4] */ {OnEOF,           OnTag,           OnTag,           OnText,   OnText, OnEndTag,  OnETag},
					   /* Tag [5] */ {OnEOF,           OnTag,           OnTag,           OnTag,    OnTag,  OnEndTag,  OnTag},
					  /* ETag [6] */ {OnEOF,           OnETag,          OnETag,          OnETag,   OnETag, OnEndETag, OnETag}};

FILE *in = NULL, *out = NULL;
char lexBuffer[MAXLEXBUFFER];
int lexBufferLength = 0;
int nLineCount = 0;
int nLineTokenCount = 0;
bool bAddNewLine = false;

CHTMLParser::CHTMLParser()
{
	ASSERT(in == NULL);
	out = NULL;
	lexBufferLength = 0;
	nLineCount = 0;
	nLineTokenCount = 0;
	bAddNewLine = false;
}

bool CHTMLParser::Open(LPCTSTR input)
{
	ASSERT(in == NULL);
	if((in = fopen(input, "rt")) == NULL) 
	{
		ErrorMessage("Error al abrir el archivo de entrada");
		return false;
	}
	/*TOKEN token;

	while (GetToken(&token) != TK_EOF) 
	{
		PrintOutput(token.tokenId);
	}*/

	return true;
}

void CHTMLParser::Close()
{
	ASSERT(in != NULL);
	fclose(in);
	in = NULL;
	out = NULL;
	lexBufferLength = 0;
	nLineCount = 0;
	nLineTokenCount = 0;
	bAddNewLine = false;
}

int CHTMLParser::GetEvent(int c)
{
	if (c == EOF) return 0;
	if (isalpha(c) || c == '_') return 1;
	if (isdigit(c)) return 2;
	
	switch (c) {
		case ' ':
		case '\t':
		case '\v':
		case '\f':
		case '\n': return 3;
		case '<':  return 4;
		case '>':  return 5;
		case '/':  return 6;
	}
	return 1;
}

int CHTMLParser::GetToken(TOKEN* pToken)
{
	int c, event, state = 0, token;
	int (*pf)(char);

	lexBufferLength = 0;

	do {
		c = fgetc(in);
		event = GetEvent(c);

		if (pf = pfSemRut[state][event]) 
			token = (*pf)((char)c);

		state = newState[state][event];

	} while (state != END);

	nLineTokenCount++;

	pToken->id = token;
	pToken->p = lexBuffer;

	return token;
}

void CHTMLParser::ErrorMessage(char* szMsg)
{
	MessageBox(NULL, szMsg, "Error", MB_OK | MB_ICONERROR);
}

/* Semantic Routines */
int GetSpecialChar()
{
	// TODO...
	return '&';
}
// Al leer un prrafo
int OnText(char c)
{
	if (lexBufferLength < MAXLEXBUFFER) {
		if (c == '&')
			c = GetSpecialChar();

 		lexBuffer[lexBufferLength++] = c;
	}
	
	return TK_TEXT;
}

// Al terminar de leer un prrafo
int OnEndTextByEOF(char c)
{
	lexBuffer[lexBufferLength] = '\0';

	return TK_TEXT;
}

// Al leer un <
int OnEndTextByTag(char c)
{
	//ungetc('<', in);
	//ungetc(c, in);	//it shuldnt cunsume the character
	fseek(in, -2, SEEK_CUR);
	lexBuffer[--lexBufferLength] = '\0';

	return TK_TEXT;
}

// Al leer un tag
int OnTag(char c)
{
	if (lexBufferLength < MAXLEXBUFFER)
 		lexBuffer[lexBufferLength++] = c;
	
	return TK_TAG;
}

// Al terminar de leer un Tag
int OnEndTag(char c)
{
	lexBuffer[lexBufferLength] = '\0';
	char* sz = lexBuffer + 1;

	if (!_stricmp(sz, "B")) return TK_TAG_B;
	else if (!_stricmp(sz, "I")) return TK_TAG_I;
	else if (!_stricmp(sz, "H1")) return TK_TAG_H1;
	else if (!_stricmp(sz, "P")) return TK_TAG_P;
	else if (!_stricmp(sz, "A")) return TK_TAG_A;
	
	return TK_TAG_UNKNOWN;
}

// Al leer un Etag </X>
int OnETag(char c)
{
	if (lexBufferLength < MAXLEXBUFFER)
 		lexBuffer[lexBufferLength++] = c;
	
	return TK_ETAG;
}

// Al terminar de leer un Etag
int OnEndETag(char c)
{
	lexBuffer[lexBufferLength] = '\0';
	char* sz = lexBuffer + 2;

	if (!_stricmp(sz, "B")) return TK_ETAG_B;
	else if (!_stricmp(sz, "I")) return TK_ETAG_I;
	else if (!_stricmp(sz, "H1")) return TK_ETAG_H1;
	else if (!_stricmp(sz, "P")) return TK_ETAG_P;
	else if (!_stricmp(sz, "A")) return TK_ETAG_A;
	
	return TK_ETAG_UNKNOWN;
}

// Al leer un End Of File
int OnEOF(char c)
{
	//Unespected EOF
	return TK_EOF;
}

void PrintOutput(int token)
{
	switch (token) 
	{
		case TK_TEXT: 
			fprintf(out, "%s", lexBuffer); 
			break;
		case TK_TAG_B: 
			fprintf(out, "BOLD"); 
			break;
		case TK_TAG_I: 
			fprintf(out, "ITALIC"); 
			break;
		case TK_ETAG_B: 
			fprintf(out, "/BOLD"); 
			break;
		case TK_ETAG_I: 
			fprintf(out, "/ITALIC"); 
			break;
		case TK_TAG_UNKNOWN: 
			fprintf(out, "UNKNOWN"); 
			break;
		case TK_ETAG_UNKNOWN: 
			fprintf(out, "//UNKNOWN"); 
			break;
	}
}