// HTMLViewerView.h : interface of the CHTMLViewerView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTMLVIEWERVIEW_H__C2914A3C_57A2_11D2_B43C_000000000000__INCLUDED_)
#define AFX_HTMLVIEWERVIEW_H__C2914A3C_57A2_11D2_B43C_000000000000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include <afxtempl.h>

struct CHARPOS
{
	POSITION pTokenInfo;
	int		 nIndex;

	bool operator==(const CHARPOS& rhs) const
	{ 
		return (pTokenInfo == rhs.pTokenInfo &&
			nIndex == rhs.nIndex);
	}
	bool operator!=(const CHARPOS& rhs) const
	{
		return !operator==(rhs);
	}
};

struct RELCHARPOS
{
	CHARPOS		charPos;
	POSITION	pLine;
	bool		bIsLineEnd;
	CPoint		pt;
};

struct TEXTLINE
{
	CHARPOS		first;
	CHARPOS		last;
	int			nHeight;
};

typedef CList<TEXTLINE, TEXTLINE&> PAGE;

class CHTMLViewerView : public CScrollView
{
protected: // create from serialization only
	CHTMLViewerView();
	DECLARE_DYNCREATE(CHTMLViewerView)

protected:
	int CalcLineBreak(int* pWidths, int nLen, int cx, int& x);
	void PrepareFont(CFont*, int);
	void BreakLines();
	int GetNextWidth(CHARPOS* pCharPos);
	int GetPrevWidth(CHARPOS* pCharPos);
	CHARPOS GetNextCharPos(CHARPOS charPos);
	bool IsLineFirstChar(const TEXTLINE& line, const CHARPOS& cp);
	bool IsLineLastChar(const TEXTLINE& line, const CHARPOS& cp);
	bool IsLineEnd(const TEXTLINE& line, const CHARPOS& cp);
	void UpdateCaret();
	void DrawLine(CDC* pDC);
	CPoint GetCharFromPt(const CPoint& pt, RELCHARPOS& rcp);
	void MoveCaretTo(const CPoint& pt, const RELCHARPOS& cp);

	TOKENLIST& GetTokenList()
	{
		#ifdef _DEBUG
			CHTMLViewerDoc* pDoc = GetDocument();
			ASSERT_VALID(pDoc);
			return pDoc->m_tokens;
		#endif
		#ifndef _DEBUG
			return GetDocument()->m_tokens;
		#endif
	}
	int GetTokenInfoHeight(POSITION pos)
	{
		return GetTokenList().GetAt(pos).m_nHeight;
	}
	
#ifdef _DEBUG
	void TestDrawing(CDC* pDC);
#endif

protected:
	RELCHARPOS	m_currPos;
	CPoint		m_caretPos;
	int			m_nCaretHeight;
	RELCHARPOS  m_selStart, m_selEnd;
	bool		m_bSelection;

// Attributes
public:
	CRect m_rcMargins;
	PAGE m_page;

// Operations
public:
	CHTMLViewerDoc* GetDocument();
	void PreparePage();

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CHTMLViewerView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void OnInitialUpdate(); // called first time after construct
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CHTMLViewerView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CHTMLViewerView)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSetFocus(CWnd* pOldWnd);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in HTMLViewerView.cpp
inline CHTMLViewerDoc* CHTMLViewerView::GetDocument()
   { return (CHTMLViewerDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_HTMLVIEWERVIEW_H__C2914A3C_57A2_11D2_B43C_000000000000__INCLUDED_)
